@@ -2,10 +2,10 @@
    MzScheme extension example that returns the string "Hello, world!"
    when loaded.
 
-   For 3m, compile with:
+   For the 3m GC (the default), compile with:
      mzc --xform hello.c
      mzc --3m --cc hello.3m.c
-     mzc --3m --ld hello.so hello.o
+     mzc --3m --ld hello.so hello_3m.o
    And load with
      (load-extension "hello.so") ; or "hello.dylib" for Mac OS X
                                  ; or "hello.dll" for Windows
