@@ -3,7 +3,7 @@
  * Purpose:     MrEd X Windows event loop
  * Author:      Matthew Flatt
  * Created:     1996
- * Copyright:   (c) 2004-2009 PLT Scheme Inc.
+ * Copyright:   (c) 2004-2010 PLT Scheme Inc.
  * Copyright:   (c) 1996, Matthew Flatt
  */
 
@@ -12,7 +12,7 @@
 #define Uses_XLib
 
 #include "wx_main.h"
-#include "wx_media.h"
+#include "wx_win.h"
 #include "wx_clipb.h"
 #include "scheme.h"
 
