@@ -43,26 +43,26 @@ void (*GC_push_last_roots)() = 0;
 
 # if !defined(NO_DEBUGGING)
 /* For debugging:	*/
-void GC_print_static_roots()
+void GC_print_static_roots(void)
 {
     register int i;
     size_t total = 0;
     
     for (i = 0; i < n_root_sets; i++) {
-        GC_printf2("From 0x%lx to 0x%lx ",
-        	   (unsigned long) GC_static_roots[i].r_start,
-        	   (unsigned long) GC_static_roots[i].r_end);
+        GC_printf("From %p to %p ",
+        	  GC_static_roots[i].r_start,
+        	  GC_static_roots[i].r_end);
         if (GC_static_roots[i].r_tmp) {
-            GC_printf0(" (temporary)\n");
+            GC_printf(" (temporary)\n");
         } else {
-            GC_printf0("\n");
+            GC_printf("\n");
         }
         total += GC_static_roots[i].r_end - GC_static_roots[i].r_start;
     }
-    GC_printf1("Total size: %ld\n", (unsigned long) total);
+    GC_printf("Total size: %ld\n", (unsigned long) total);
     if (GC_root_size != total) {
-     	GC_printf1("GC_root_size incorrect: %ld!!\n",
-     		   (unsigned long) GC_root_size);
+     	GC_printf("GC_root_size incorrect: %ld!!\n",
+     		  (unsigned long) GC_root_size);
     }
 }
 # endif /* NO_DEBUGGING */
@@ -70,8 +70,7 @@ void GC_print_static_roots()
 /* Primarily for debugging support:	*/
 /* Is the address p in one of the registered static			*/
 /* root sections?							*/
-GC_bool GC_is_static_root(p)
-ptr_t p;
+GC_bool GC_is_static_root(ptr_t p)
 {
     static int last_root_set = MAX_ROOT_SETS;
     register int i;
@@ -101,8 +100,7 @@ ptr_t p;
 	-- really defined in gc_priv.h
 */
 
-static int rt_hash(addr)
-char * addr;
+static INLINE int rt_hash(ptr_t addr)
 {
     word result = (word) addr;
 #   if CPP_WORDSZ > 8*LOG_RT_SIZE
@@ -119,11 +117,10 @@ char * addr;
 
 /* Is a range starting at b already in the table? If so return a	*/
 /* pointer to it, else NIL.						*/
-struct roots * GC_roots_present(b)
-char *b;
+struct roots * GC_roots_present(ptr_t b)
 {
-    register int h = rt_hash(b);
-    register struct roots *p = GC_root_index[h];
+    int h = rt_hash(b);
+    struct roots *p = GC_root_index[h];
     
     while (p != 0) {
         if (p -> r_start == (ptr_t)b) return(p);
@@ -133,10 +130,9 @@ char *b;
 }
 
 /* Add the given root structure to the index. */
-static void add_roots_to_index(p)
-struct roots *p;
+static void add_roots_to_index(struct roots *p)
 {
-    register int h = rt_hash(p -> r_start);
+    int h = rt_hash(p -> r_start);
     
     p -> r_next = GC_root_index[h];
     GC_root_index[h] = p;
@@ -153,16 +149,14 @@ struct roots *p;
 
 word GC_root_size = 0;
 
-void GC_add_roots(b, e)
-char * b; char * e;
+void GC_add_roots(void *b, void *e)
 {
     DCL_LOCK_STATE;
     
-    DISABLE_SIGNALS();
+    if (!GC_is_initialized) GC_init();
     LOCK();
-    GC_add_roots_inner(b, e, FALSE);
+    GC_add_roots_inner((ptr_t)b, (ptr_t)e, FALSE);
     UNLOCK();
-    ENABLE_SIGNALS();
 }
 
 
@@ -172,9 +166,7 @@ char * b; char * e;
 /* them correctly.)							*/
 /* Tmp specifies that the interval may be deleted before 		*/
 /* reregistering dynamic libraries.					*/ 
-void GC_add_roots_inner(b, e, tmp)
-char * b; char * e;
-GC_bool tmp;
+void GC_add_roots_inner(ptr_t b, ptr_t e, GC_bool tmp)
 {
     struct roots * old;
     
@@ -191,15 +183,15 @@ GC_bool tmp;
         
         for (i = 0; i < n_root_sets; i++) {
             old = GC_static_roots + i;
-            if ((ptr_t)b <= old -> r_end && (ptr_t)e >= old -> r_start
+            if (b <= old -> r_end && e >= old -> r_start
 		&& tmp == old -> r_tmp) { /* PLTSCHEME: merge only same kinds */
-                if ((ptr_t)b < old -> r_start) {
-                    old -> r_start = (ptr_t)b;
-                    GC_root_size += (old -> r_start - (ptr_t)b);
+                if (b < old -> r_start) {
+                    old -> r_start = b;
+                    GC_root_size += (old -> r_start - b);
                 }
-                if ((ptr_t)e > old -> r_end) {
-                    old -> r_end = (ptr_t)e;
-                    GC_root_size += ((ptr_t)e - old -> r_end);
+                if (e > old -> r_end) {
+                    old -> r_end = e;
+                    GC_root_size += (e - old -> r_end);
                 }
                 old -> r_tmp &= tmp;
                 break;
@@ -211,25 +203,25 @@ GC_bool tmp;
             
             for (i++; i < n_root_sets; i++) {
               other = GC_static_roots + i;
-              b = (char *)(other -> r_start);
-              e = (char *)(other -> r_end);
+              b = other -> r_start;
+              e = other -> r_end;
 	      tmp = other -> r_tmp; /* PLTSCHEME */
-              if ((ptr_t)b <= old -> r_end && (ptr_t)e >= old -> r_start
-		  && tmp == old -> r_tmp) {  /* PLTSCHEME: merge only same kinds */
-                if ((ptr_t)b < old -> r_start) {
-                    old -> r_start = (ptr_t)b;
-                    GC_root_size += (old -> r_start - (ptr_t)b);
+              if (b <= old -> r_end && e >= old -> r_start
+                  && tmp == old -> r_tmp) {  /* PLTSCHEME: merge only same kinds */
+                if (b < old -> r_start) {
+                    old -> r_start = b;
+                    GC_root_size += (old -> r_start - b);
                 }
-                if ((ptr_t)e > old -> r_end) {
-                    old -> r_end = (ptr_t)e;
-                    GC_root_size += ((ptr_t)e - old -> r_end);
+                if (e > old -> r_end) {
+                    old -> r_end = e;
+                    GC_root_size += (e - old -> r_end);
                 }
                 old -> r_tmp &= other -> r_tmp;
                 /* Delete this entry. */
                   GC_root_size -= (other -> r_end - other -> r_start);
                   other -> r_start = GC_static_roots[n_root_sets-1].r_start;
                   other -> r_end = GC_static_roots[n_root_sets-1].r_end;
-                                  n_root_sets--;
+                  n_root_sets--;
               }
             }
           return;
@@ -238,10 +230,10 @@ GC_bool tmp;
 #   else
       old = GC_roots_present(b);
       if (old != 0) {
-        if ((ptr_t)e <= old -> r_end) /* already there */ return;
+        if (e <= old -> r_end) /* already there */ return;
         /* else extend */
-        GC_root_size += (ptr_t)e - old -> r_end;
-        old -> r_end = (ptr_t)e;
+        GC_root_size += e - old -> r_end;
+        old -> r_end = e;
         return;
       }
 #   endif
@@ -255,17 +247,17 @@ GC_bool tmp;
       GC_static_roots[n_root_sets].r_next = 0;
 #   endif
     add_roots_to_index(GC_static_roots + n_root_sets);
-    GC_root_size += (ptr_t)e - (ptr_t)b;
+    GC_root_size += e - b;
     n_root_sets++;
 }
 
 static GC_bool roots_were_cleared = FALSE;
 
-void GC_clear_roots GC_PROTO((void))
+void GC_clear_roots (void)
 {
     DCL_LOCK_STATE;
     
-    DISABLE_SIGNALS();
+    if (!GC_is_initialized) GC_init();
     LOCK();
     roots_were_cleared = TRUE;
     n_root_sets = 0;
@@ -278,12 +270,10 @@ void GC_clear_roots GC_PROTO((void))
     }
 #   endif
     UNLOCK();
-    ENABLE_SIGNALS();
 }
 
 /* Internal use only; lock held.	*/
-static void GC_remove_root_at_pos(i) 
-int i;
+static void GC_remove_root_at_pos(int i) 
 {
     GC_root_size -= (GC_static_roots[i].r_end - GC_static_roots[i].r_start);
     GC_static_roots[i].r_start = GC_static_roots[n_root_sets-1].r_start;
@@ -293,9 +283,9 @@ int i;
 }
 
 #if !defined(MSWIN32) && !defined(MSWINCE)
-static void GC_rebuild_root_index()
+static void GC_rebuild_root_index(void)
 {
-    register int i;
+    int i;
     	
     for (i = 0; i < RT_SIZE; i++) GC_root_index[i] = 0;
     for (i = 0; i < n_root_sets; i++)
@@ -304,16 +294,16 @@ static void GC_rebuild_root_index()
 #endif
 
 /* Internal use only; lock held.	*/
-void GC_remove_tmp_roots()
+void GC_remove_tmp_roots(void)
 {
-    register int i;
+    int i;
     
     for (i = 0; i < n_root_sets; ) {
     	if (GC_static_roots[i].r_tmp) {
             GC_remove_root_at_pos(i);
     	} else {
     	    i++;
-    }
+	}
     }
     #if !defined(MSWIN32) && !defined(MSWINCE)
     GC_rebuild_root_index();
@@ -321,25 +311,22 @@ void GC_remove_tmp_roots()
 }
 
 #if !defined(MSWIN32) && !defined(MSWINCE)
-void GC_remove_roots(b, e)
-char * b; char * e;
+void GC_remove_roots(void *b, void *e)
 {
     DCL_LOCK_STATE;
     
-    DISABLE_SIGNALS();
     LOCK();
-    GC_remove_roots_inner(b, e);
+    GC_remove_roots_inner((ptr_t)b, (ptr_t)e);
     UNLOCK();
-    ENABLE_SIGNALS();
 }
 
 /* Should only be called when the lock is held */
-void GC_remove_roots_inner(b,e)
-char * b; char * e;
+void GC_remove_roots_inner(ptr_t b, ptr_t e)
 {
     int i;
     for (i = 0; i < n_root_sets; ) {
-    	if (GC_static_roots[i].r_start >= (ptr_t)b && GC_static_roots[i].r_end <= (ptr_t)e) {
+    	if (GC_static_roots[i].r_start >= b
+	    && GC_static_roots[i].r_end <= e) {
             GC_remove_root_at_pos(i);
     	} else {
     	    i++;
@@ -352,8 +339,7 @@ char * b; char * e;
 #if defined(MSWIN32) || defined(_WIN32_WCE_EMULATION)
 /* Workaround for the OS mapping and unmapping behind our back:		*/
 /* Is the address p in one of the temporary static root sections?	*/
-GC_bool GC_is_tmp_root(p)
-ptr_t p;
+GC_bool GC_is_tmp_root(ptr_t p)
 {
     static int last_root_set = MAX_ROOT_SETS;
     register int i;
@@ -373,9 +359,9 @@ ptr_t p;
 }
 #endif /* MSWIN32 || _WIN32_WCE_EMULATION */
 
-ptr_t GC_approx_sp()
+ptr_t GC_approx_sp(void)
 {
-    VOLATILE word dummy;
+    volatile word dummy;
 
     dummy = 42;	/* Force stack to grow if necessary.	Otherwise the	*/
     		/* later accesses might cause the kernel to think we're	*/
@@ -408,8 +394,7 @@ size_t GC_excl_table_entries = 0;	/* Number of entries in use.	  */
 /* Return the first exclusion range that includes an address >= start_addr */
 /* Assumes the exclusion table contains at least one entry (namely the	   */
 /* GC data structures).							   */
-struct exclusion * GC_next_exclusion(start_addr)
-ptr_t start_addr;
+struct exclusion * GC_next_exclusion(ptr_t start_addr)
 {
     size_t low = 0;
     size_t high = GC_excl_table_entries - 1;
@@ -428,9 +413,7 @@ ptr_t start_addr;
     return GC_excl_table + low;
 }
 
-void GC_exclude_static_roots(start, finish)
-GC_PTR start;
-GC_PTR finish;
+void GC_exclude_static_roots(void *start, void *finish)
 {
     struct exclusion * next;
     size_t next_index, i;
@@ -464,10 +447,7 @@ GC_PTR finish;
 }
 
 /* Invoke push_conditional on ranges that are not excluded. */
-void GC_push_conditional_with_exclusions(bottom, top, all)
-ptr_t bottom;
-ptr_t top;
-int all;
+void GC_push_conditional_with_exclusions(ptr_t bottom, ptr_t top, GC_bool all)
 {
     struct exclusion * next;
     ptr_t excl_start;
@@ -488,9 +468,9 @@ int all;
  * In the presence of threads, push enough of the current stack
  * to ensure that callee-save registers saved in collector frames have been
  * seen.
+ * FIXME: Merge with per-thread stuff.
  */
-void GC_push_current_stack(cold_gc_frame)
-ptr_t cold_gc_frame;
+void GC_push_current_stack(ptr_t cold_gc_frame, void * context)
 {
 #   if defined(THREADS)
 	if (0 == cold_gc_frame) return;
@@ -538,27 +518,30 @@ ptr_t cold_gc_frame;
 #   endif /* !THREADS */
 }
 
+void (*GC_push_typed_structures) (void) = NULL;
+
 /*
  * Push GC internal roots.  Only called if there is some reason to believe
  * these would not otherwise get registered.
  */
-void GC_push_gc_structures GC_PROTO((void))
+void GC_push_gc_structures(void)
 {
     GC_push_finalizer_structures();
-    GC_push_stubborn_structures();
 #   if defined(THREADS)
       GC_push_thread_structures();
 #   endif
+    if( GC_push_typed_structures )
+      GC_push_typed_structures();
 }
 
 #ifdef THREAD_LOCAL_ALLOC
-  void GC_mark_thread_local_free_lists();
+  void GC_mark_thread_local_free_lists(void);
 #endif
 
-void GC_cond_register_dynamic_libraries()
+void GC_cond_register_dynamic_libraries(void)
 {
-# if (defined(DYNAMIC_LOADING) || defined(MSWIN32) || defined(MSWINCE) \
-     || defined(PCR)) && !defined(SRC_M3)
+# if defined(DYNAMIC_LOADING) || defined(MSWIN32) || defined(MSWINCE) \
+     || defined(PCR)
     GC_remove_tmp_roots();
     if (!GC_no_dls) GC_register_dynamic_libraries();
 # else
@@ -575,12 +558,10 @@ void GC_cond_register_dynamic_libraries()
  * A zero value indicates that it's OK to miss some
  * register values.
  */
-void GC_push_roots(all, cold_gc_frame)
-GC_bool all;
-ptr_t cold_gc_frame;
+void GC_push_roots(GC_bool all, ptr_t cold_gc_frame)
 {
     int i;
-    int kind;
+    unsigned kind;
 
     /*
      * Next push static data.  This must happen early on, since it's
@@ -608,7 +589,7 @@ ptr_t cold_gc_frame;
      /* saves us the trouble of scanning them, and possibly that of	*/
      /* marking the freelists.						*/
        for (kind = 0; kind < GC_n_kinds; kind++) {
-	 GC_PTR base = GC_base(GC_obj_kinds[kind].ok_freelist);
+	 void *base = GC_base(GC_obj_kinds[kind].ok_freelist);
 	 if (0 != base) {
 	   GC_set_mark_bit(base);
 	 }
@@ -625,7 +606,7 @@ ptr_t cold_gc_frame;
      /* If the world is not stopped, this is unsafe.  It is	*/
      /* also unnecessary, since we will do this again with the	*/
      /* world stopped.						*/
-#      ifdef THREAD_LOCAL_ALLOC
+#      if defined(THREAD_LOCAL_ALLOC)
          if (GC_world_stopped) GC_mark_thread_local_free_lists();
 #      endif
 
@@ -633,23 +614,11 @@ ptr_t cold_gc_frame;
      * Now traverse stacks, and mark from register contents.
      * These must be done last, since they can legitimately overflow
      * the mark stack.
+     * This is usually done by saving the current context on the
+     * stack, and then just tracing from the stack.
      */
-#   ifdef USE_GENERIC_PUSH_REGS
-	GC_generic_push_regs(cold_gc_frame);
-	/* Also pushes stack, so that we catch callee-save registers	*/
-	/* saved inside the GC_push_regs frame.				*/
-#   else
-       /*
-        * push registers - i.e., call GC_push_one(r) for each
-        * register contents r.
-        */
-        GC_push_regs(); /* usually defined in machine_dep.c */
-	GC_push_current_stack(cold_gc_frame);
-	/* In the threads case, this only pushes collector frames.      */
-	/* In the case of linux threads on IA64, the hot section of	*/
-	/* the main stack is marked here, but the register stack	*/
-	/* backing store is handled in the threads-specific code.	*/
-#   endif
+      GC_push_regs_and_stack(cold_gc_frame);
+
     if (GC_push_other_roots != 0) (*GC_push_other_roots)();
     	/* In the threads case, this also pushes thread stacks.	*/
         /* Note that without interior pointer recognition lots	*/
@@ -665,4 +634,3 @@ void GC_flush_mark_stack()
 {
   while (!GC_mark_stack_empty()) GC_mark_from_mark_stack();
 }
-
