@@ -1,6 +1,6 @@
 /*
   MzScheme
-  Copyright (c) 2004-2009 PLT Scheme Inc.
+  Copyright (c) 2004-2010 PLT Scheme Inc.
   Copyright (c) 1995-2001 Matthew Flatt
 
     This library is free software; you can redistribute it and/or
@@ -56,11 +56,8 @@ Scheme_Object *scheme_real_to_complex(const Scheme_Object *n)
   return make_complex(n, zero, 0);
 }
 
-#ifdef MZ_XFORM
-START_XFORM_SKIP;
-#endif
-
 Scheme_Object *scheme_make_small_complex(const Scheme_Object *n, Small_Complex *s)
+  XFORM_SKIP_PROC
 {
   s->so.type = scheme_complex_type;
   s->r = (Scheme_Object *)n;
@@ -69,10 +66,6 @@ Scheme_Object *scheme_make_small_complex(const Scheme_Object *n, Small_Complex *
   return (Scheme_Object *)s;
 }
 
-#ifdef MZ_XFORM
-END_XFORM_SKIP;
-#endif
-
 int scheme_is_complex_exact(const Scheme_Object *o)
 {
   Scheme_Complex *c = (Scheme_Complex *)o;
