@@ -441,6 +441,8 @@ static jit_state 			_jit;
 #define jit_finish(sub)			jit_calli(sub)
 #endif
 
+#define jit_normal_finish(sub)          jit_finish(sub)
+
 #ifndef jit_finishr
 #define jit_finishr(reg)		jit_callr(reg)
 #endif
@@ -454,7 +456,6 @@ static jit_state 			_jit;
 #endif
 
 #ifndef jit_getarg_c
-#ifndef JIT_FP
 #define jit_getarg_c(reg, ofs)		jit_extr_c_i  ((reg), (ofs))
 #define jit_getarg_i(reg, ofs)		jit_movr_i    ((reg), (ofs))
 #define jit_getarg_l(reg, ofs)		jit_movr_l    ((reg), (ofs))
@@ -464,17 +465,6 @@ static jit_state 			_jit;
 #define jit_getarg_ui(reg, ofs)		jit_movr_ui   ((reg), (ofs))
 #define jit_getarg_ul(reg, ofs)		jit_extr_uc_ul((reg), (ofs))
 #define jit_getarg_us(reg, ofs)		jit_extr_us_ul((reg), (ofs))
-#else
-#define jit_getarg_c(reg, ofs)		jit_ldxi_c((reg), JIT_FP, (ofs));
-#define jit_getarg_uc(reg, ofs)		jit_ldxi_uc((reg), JIT_FP, (ofs));
-#define jit_getarg_s(reg, ofs)		jit_ldxi_s((reg), JIT_FP, (ofs));
-#define jit_getarg_us(reg, ofs)		jit_ldxi_us((reg), JIT_FP, (ofs));
-#define jit_getarg_i(reg, ofs)		jit_ldxi_i((reg), JIT_FP, (ofs));
-#define jit_getarg_ui(reg, ofs)		jit_ldxi_ui((reg), JIT_FP, (ofs));
-#define jit_getarg_l(reg, ofs)		jit_ldxi_l((reg), JIT_FP, (ofs));
-#define jit_getarg_ul(reg, ofs)		jit_ldxi_ul((reg), JIT_FP, (ofs));
-#define jit_getarg_p(reg, ofs)		jit_ldxi_p((reg), JIT_FP, (ofs));
-#endif
 #endif
 
 
