@@ -1,6 +1,6 @@
 /*
   MzScheme
-  Copyright (c) 2004-2009 PLT Scheme Inc.
+  Copyright (c) 2004-2010 PLT Scheme Inc.
   Copyright (c) 1995-2001 Matthew Flatt
 
     This library is free software; you can redistribute it and/or
@@ -25,13 +25,16 @@
 
 #include "schpriv.h"
 
-/* globals */
-Scheme_Object scheme_null[1];
-Scheme_Object *scheme_cons_proc;
-Scheme_Object *scheme_mcons_proc;
-Scheme_Object *scheme_list_proc;
-Scheme_Object *scheme_list_star_proc;
-Scheme_Object *scheme_box_proc;
+/* read only globals */
+READ_ONLY Scheme_Object scheme_null[1];
+READ_ONLY Scheme_Object *scheme_cons_proc;
+READ_ONLY Scheme_Object *scheme_mcons_proc;
+READ_ONLY Scheme_Object *scheme_list_proc;
+READ_ONLY Scheme_Object *scheme_list_star_proc;
+READ_ONLY Scheme_Object *scheme_box_proc;
+/* read only locals */
+ROSYM static Scheme_Object *weak_symbol;
+ROSYM static Scheme_Object *equal_symbol;
 
 /* locals */
 static Scheme_Object *pair_p_prim (int argc, Scheme_Object *argv[]);
@@ -135,13 +138,20 @@ static Scheme_Object *make_hasheq_placeholder(int argc, Scheme_Object *argv[]);
 static Scheme_Object *make_hasheqv_placeholder(int argc, Scheme_Object *argv[]);
 static Scheme_Object *table_placeholder_p(int argc, Scheme_Object *argv[]);
 
+static Scheme_Object *unsafe_car (int argc, Scheme_Object *argv[]);
+static Scheme_Object *unsafe_cdr (int argc, Scheme_Object *argv[]);
+static Scheme_Object *unsafe_mcar (int argc, Scheme_Object *argv[]);
+static Scheme_Object *unsafe_mcdr (int argc, Scheme_Object *argv[]);
+static Scheme_Object *unsafe_set_mcar (int argc, Scheme_Object *argv[]);
+static Scheme_Object *unsafe_set_mcdr (int argc, Scheme_Object *argv[]);
+static Scheme_Object *unsafe_unbox (int argc, Scheme_Object *argv[]);
+static Scheme_Object *unsafe_set_box (int argc, Scheme_Object *argv[]);
+
 #define BOX "box"
 #define BOXP "box?"
 #define UNBOX "unbox"
 #define SETBOX "set-box!"
 
-static Scheme_Object *weak_symbol, *equal_symbol;
-
 void
 scheme_init_list (Scheme_Env *env)
 {
@@ -684,6 +694,46 @@ scheme_init_list (Scheme_Env *env)
   equal_symbol = scheme_intern_symbol("equal");
 }
 
+void
+scheme_init_unsafe_list (Scheme_Env *env)
+{
+  Scheme_Object *p;
+  
+  scheme_null->type = scheme_null_type;
+
+  p = scheme_make_folding_prim(unsafe_car, "unsafe-car", 1, 1, 1);
+  SCHEME_PRIM_PROC_FLAGS(p) |= SCHEME_PRIM_IS_UNARY_INLINED;
+  scheme_add_global_constant ("unsafe-car", p, env);
+
+  p = scheme_make_folding_prim(unsafe_cdr, "unsafe-cdr", 1, 1, 1);
+  SCHEME_PRIM_PROC_FLAGS(p) |= SCHEME_PRIM_IS_UNARY_INLINED;
+  scheme_add_global_constant ("unsafe-cdr", p, env);
+
+  p = scheme_make_immed_prim(unsafe_mcar, "unsafe-mcar", 1, 1);
+  SCHEME_PRIM_PROC_FLAGS(p) |= SCHEME_PRIM_IS_UNARY_INLINED;
+  scheme_add_global_constant ("unsafe-mcar", p, env);
+
+  p = scheme_make_immed_prim(unsafe_mcdr, "unsafe-mcdr", 1, 1);
+  SCHEME_PRIM_PROC_FLAGS(p) |= SCHEME_PRIM_IS_UNARY_INLINED;
+  scheme_add_global_constant ("unsafe-mcdr", p, env);
+
+  p = scheme_make_immed_prim(unsafe_set_mcar, "unsafe-set-mcar!", 2, 2);
+  SCHEME_PRIM_PROC_FLAGS(p) |= SCHEME_PRIM_IS_BINARY_INLINED;
+  scheme_add_global_constant ("unsafe-set-mcar!", p, env);
+
+  p = scheme_make_immed_prim(unsafe_set_mcdr, "unsafe-set-mcdr!", 2, 2);
+  SCHEME_PRIM_PROC_FLAGS(p) |= SCHEME_PRIM_IS_BINARY_INLINED;
+  scheme_add_global_constant ("unsafe-set-mcdr!", p, env);
+  
+  p = scheme_make_immed_prim(unsafe_unbox, "unsafe-unbox", 1, 1);
+  SCHEME_PRIM_PROC_FLAGS(p) |= SCHEME_PRIM_IS_UNARY_INLINED;  
+  scheme_add_global_constant("unsafe-unbox", p, env);
+
+  p = scheme_make_immed_prim(unsafe_set_box, "unsafe-set-box!", 2, 2);
+  SCHEME_PRIM_PROC_FLAGS(p) |= SCHEME_PRIM_IS_BINARY_INLINED;
+  scheme_add_global_constant("unsafe-set-box!", p, env);
+}
+
 Scheme_Object *scheme_make_pair(Scheme_Object *car, Scheme_Object *cdr)
 {
 #ifdef MZ_PRECISE_GC
@@ -2534,3 +2584,50 @@ void scheme_init_ephemerons(void)
 }
 
 #endif
+
+/************************************************************/
+/*                        unsafe                            */
+/************************************************************/
+
+static Scheme_Object *unsafe_car (int argc, Scheme_Object *argv[])
+{
+  return SCHEME_CAR(argv[0]);
+}
+
+static Scheme_Object *unsafe_cdr (int argc, Scheme_Object *argv[])
+{
+  return SCHEME_CDR(argv[0]);
+}
+
+static Scheme_Object *unsafe_mcar (int argc, Scheme_Object *argv[])
+{
+  return SCHEME_CAR(argv[0]);
+}
+
+static Scheme_Object *unsafe_mcdr (int argc, Scheme_Object *argv[])
+{
+  return SCHEME_CDR(argv[0]);
+}
+
+static Scheme_Object *unsafe_set_mcar (int argc, Scheme_Object *argv[])
+{
+  SCHEME_CAR(argv[0]) = argv[1];
+  return scheme_void;
+}
+
+static Scheme_Object *unsafe_set_mcdr (int argc, Scheme_Object *argv[])
+{
+  SCHEME_CDR(argv[0]) = argv[1];
+  return scheme_void;
+}
+
+static Scheme_Object *unsafe_unbox (int argc, Scheme_Object *argv[])
+{
+  return SCHEME_BOX_VAL(argv[0]);
+}
+
+static Scheme_Object *unsafe_set_box (int argc, Scheme_Object *argv[])
+{
+  SCHEME_BOX_VAL(argv[0]) = argv[1];
+  return scheme_void;
+}
