@@ -35,6 +35,12 @@ START_XFORM_SUSPEND;
 # endif
 #endif
 
+#ifndef MZ_PRECISE_GC
+int GC_pthread_join(pthread_t thread, void **retval);
+int GC_pthread_create(pthread_t *thread, const pthread_attr_t *attr, void *(*start_routine)(void*), void * arg);
+int GC_pthread_detach(pthread_t thread);
+#endif
+
 void mzrt_set_user_break_handler(void (*user_break_handler)(int))
 {
 #ifdef WIN32
@@ -66,7 +72,8 @@ static void rungdb() {
       case 'd':
         snprintf(outbuffer, 100, "xterm -e gdb ./mzscheme3m %d &", pid);
         fprintf(stderr, "%s\n", outbuffer);
-        system(outbuffer);
+        if(system(outbuffer)) 
+          fprintf(stderr, "system failed\n");
         break;
       case 'e':
       default:
@@ -156,6 +163,7 @@ void *mzrt_thread_stub(void *data){
   mzrt_thread_stub_data *stub_data  = (mzrt_thread_stub_data*) data;
   void * (*start_proc)(void *)        = stub_data->start_proc;
   void *start_proc_data               = stub_data->data;
+  scheme_init_os_thread();
   proc_thread_self                    = stub_data->thread;
 
   free(data);
@@ -163,17 +171,17 @@ void *mzrt_thread_stub(void *data){
   return start_proc(start_proc_data);
 }
 
-unsigned int mz_proc_thread_self() {
+mzrt_thread_id mz_proc_thread_self() {
 #ifdef WIN32
 #error !!!mz_proc_thread_id not implemented!!!
 #else
-  return (unsigned int) pthread_self();
+  return pthread_self();
 #endif
 }
 
-unsigned int mz_proc_thread_id(mz_proc_thread* thread) {
+mzrt_thread_id mz_proc_thread_id(mz_proc_thread* thread) {
 
-  return (unsigned int) thread->threadid;
+  return thread->threadid;
 }
 
 mz_proc_thread* mzrt_proc_first_thread_init() {
@@ -188,24 +196,28 @@ mz_proc_thread* mzrt_proc_first_thread_init() {
 
 mz_proc_thread* mz_proc_thread_create(mz_proc_thread_start start_proc, void* data) {
   mz_proc_thread *thread = (mz_proc_thread*)malloc(sizeof(mz_proc_thread));
-#ifdef MZ_PRECISE_GC
+  pthread_attr_t *attr;
+
+#ifdef OS_X
+  pthread_attr_t attr_storage;
+  attr = &attr_storage;
+  pthread_attr_init(attr);
+  pthread_attr_setstacksize(attr, 8*1024*1024); /*8MB*/
+#else
+  attr = NULL;
+#endif
+
   mzrt_thread_stub_data *stub_data = (mzrt_thread_stub_data*)malloc(sizeof(mzrt_thread_stub_data));
   thread->mbox = pt_mbox_create();
   stub_data->start_proc = start_proc;
   stub_data->data       = data;
   stub_data->thread     = thread;
 #   ifdef WIN32
-  thread->threadid = CreateThread(NULL, 0, start_proc, data, 0, NULL);
+  thread->threadid = CreateThread(NULL, 0, mzrt_thread_stub, stub_data, 0, NULL);
 #   else
-  pthread_create(&thread->threadid, NULL, mzrt_thread_stub, stub_data);
+  pthread_create(&thread->threadid, attr, mzrt_thread_stub, stub_data);
 #   endif
-#else
-#   ifdef WIN32
-  thread->threadid = GC_CreateThread(NULL, 0, start_proc, data, 0, NULL);
-#   else
-  GC_pthread_create(&thread->threadid, NULL, start_proc, data);
-#   endif
-#endif
+
   return thread;
 }
 
@@ -226,6 +238,21 @@ void * mz_proc_thread_wait(mz_proc_thread *thread) {
 #endif
 }
 
+int mz_proc_thread_detach(mz_proc_thread *thread) {
+#ifdef WIN32
+  DWORD rc;
+  return (void *) rc;
+#else
+  int rc;
+#   ifndef MZ_PRECISE_GC
+  rc = GC_pthread_detach(thread->threadid);
+#   else
+  rc = pthread_detach(thread->threadid);
+#   endif
+  return rc;
+#endif
+}
+
 /***********************************************************************/
 /*                RW Lock                                              */
 /***********************************************************************/
@@ -327,6 +354,66 @@ int mzrt_cond_destroy(mzrt_cond *cond) {
   return pthread_cond_destroy(&cond->cond);
 }
 
+struct mzrt_sema {
+  int ready;
+  pthread_mutex_t m;
+  pthread_cond_t c;
+};
+
+int mzrt_sema_create(mzrt_sema **_s, int v)
+{
+  mzrt_sema *s;
+  int err;
+
+  s = (mzrt_sema *)malloc(sizeof(mzrt_sema));
+  err = pthread_mutex_init(&s->m, NULL);
+  if (err) { 
+    free(s); 
+    return err; 
+  }
+  err = pthread_cond_init(&s->c, NULL);
+  if (err) { 
+    pthread_mutex_destroy(&s->m);
+    free(s); 
+    return err; 
+  }
+  s->ready = v;
+  *_s = s;
+
+  return 0;
+}
+
+int mzrt_sema_wait(mzrt_sema *s)
+{
+  pthread_mutex_lock(&s->m);
+  while (!s->ready) {
+    pthread_cond_wait(&s->c, &s->m);
+  }
+  --s->ready;
+  pthread_mutex_unlock(&s->m);
+
+  return 0;
+}
+
+int mzrt_sema_post(mzrt_sema *s)
+{
+  pthread_mutex_lock(&s->m);
+  s->ready++;
+  pthread_cond_signal(&s->c);
+  pthread_mutex_unlock(&s->m);
+
+  return 0;
+}
+
+int mzrt_sema_destroy(mzrt_sema *s)
+{
+  pthread_mutex_destroy(&s->m);
+  pthread_cond_destroy(&s->c);
+  free(s);
+
+  return 0;
+}
+
 /****************** PROCESS THREAD MAIL BOX *******************************/
 
 pt_mbox *pt_mbox_create() {
