@@ -1,6 +1,6 @@
 /*
   MzScheme
-  Copyright (c) 2004-2009 PLT Scheme Inc.
+  Copyright (c) 2004-2010 PLT Scheme Inc.
   Copyright (c) 1995-2001 Matthew Flatt
  
     This library is free software; you can redistribute it and/or
@@ -23,8 +23,8 @@
 
 #ifndef NO_SCHEME_THREADS
 
-Scheme_Object *scheme_always_ready_evt;
-Scheme_Object *scheme_system_idle_channel;
+READ_ONLY Scheme_Object *scheme_always_ready_evt;
+THREAD_LOCAL_DECL(Scheme_Object *scheme_system_idle_channel);
 
 static Scheme_Object *make_sema(int n, Scheme_Object **p);
 static Scheme_Object *semap(int n, Scheme_Object **p);
@@ -57,10 +57,9 @@ static int thread_recv_ready(Scheme_Object *ch, Scheme_Schedule_Info *sinfo);
 
 static int pending_break(Scheme_Thread *p);
 
-int scheme_main_was_once_suspended;
-
-static Scheme_Object *system_idle_put_evt;
-static Scheme_Object *thread_recv_evt;
+THREAD_LOCAL_DECL(int scheme_main_was_once_suspended);
+THREAD_LOCAL_DECL(static Scheme_Object *system_idle_put_evt);
+READ_ONLY static Scheme_Object *thread_recv_evt;
 
 #ifdef MZ_PRECISE_GC
 static void register_traversers(void);
@@ -196,9 +195,6 @@ void scheme_init_sema(Scheme_Env *env)
   o->type = scheme_thread_recv_evt_type;
   thread_recv_evt = o;
 
-  REGISTER_SO(scheme_system_idle_channel);
-  scheme_system_idle_channel = scheme_make_channel();
-
   scheme_add_evt(scheme_sema_type, sema_ready, NULL, NULL, 0);
   scheme_add_evt_through_sema(scheme_semaphore_repost_type, sema_for_repost, NULL);
   scheme_add_evt(scheme_channel_type, (Scheme_Ready_Fun)channel_get_ready, NULL, NULL, 1);
@@ -210,6 +206,11 @@ void scheme_init_sema(Scheme_Env *env)
   scheme_add_evt(scheme_thread_recv_evt_type, (Scheme_Ready_Fun)thread_recv_ready, NULL, NULL, 0);
 }
 
+void scheme_init_sema_places() {
+  REGISTER_SO(scheme_system_idle_channel);
+  scheme_system_idle_channel = scheme_make_channel();
+}
+
 Scheme_Object *scheme_make_sema(long v)
 {
   Scheme_Sema *sema;
